@@ -61,69 +61,44 @@ setClass("GAlignmentsList",
 ###
 
 setMethod("seqnames", "GAlignmentsList", 
-    function(x) 
-        new2("CompressedRleList",
-             unlistData=x@unlistData@seqnames, 
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(seqnames(unlist(x, use.names=FALSE)), x)
 )
 
 setMethod("rname", "GAlignmentsList", 
-    function(x) 
-        new2("CompressedRleList",
-             unlistData=x@unlistData@seqnames, 
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(rname(unlist(x, use.names=FALSE)), x)
 )
 
 setMethod("cigar", "GAlignmentsList", 
-    function(x) 
-        new2("CompressedCharacterList",
-             unlistData=x@unlistData@cigar, 
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(cigar(unlist(x, use.names=FALSE)), x)
 )
 
 setMethod("strand", "GAlignmentsList",
-    function(x)
-        new2("CompressedRleList",
-             unlistData=x@unlistData@strand, 
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(strand(unlist(x, use.names=FALSE)), x)
 )
 
 setMethod("qwidth", "GAlignmentsList",
-    function(x)
-        new2("CompressedIntegerList",
-             unlistData=cigarWidthAlongQuerySpace(x@unlistData@cigar),
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(qwidth(unlist(x, use.names=FALSE)), x)
 )
 
 setMethod("njunc", "GAlignmentsList",
-    function(x)
-        new2("CompressedIntegerList",
-             unlistData=unname(elementLengths(rglist(x@unlistData))) - 1L,
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(njunc(unlist(x, use.names=FALSE)), x)
 )
 
 setMethod("start", "GAlignmentsList",
-    function(x, ...)
-        new2("CompressedIntegerList",
-             unlistData=x@unlistData@start,
-             partitioning=x@partitioning, check=FALSE)
+    function(x, ...) relist(start(unlist(x, use.names=FALSE), ...), x)
 )
 
 setMethod("end", "GAlignmentsList",
-    function(x, ...)
-        new2("CompressedIntegerList",
-             unlistData=end(x@unlistData),
-             partitioning=x@partitioning, check=FALSE)
+    function(x, ...) relist(end(unlist(x, use.names=FALSE), ...), x)
 )
 
 setMethod("width", "GAlignmentsList",
-    function(x)
-        new2("CompressedIntegerList",
-             unlistData=cigarWidthAlongReferenceSpace(x@unlistData@cigar),
-             partitioning=x@partitioning, check=FALSE)
+    function(x) relist(width(unlist(x, use.names=FALSE)), x)
 )
 
-setMethod("seqinfo", "GAlignmentsList", function(x) seqinfo(x@unlistData))
+setMethod("seqinfo", "GAlignmentsList",
+    function(x) seqinfo(unlist(x, use.names=FALSE))
+)
 
 setMethod("elementMetadata", "GAlignmentsList",
     GenomicRanges:::getElementMetadataList
@@ -144,6 +119,15 @@ setReplaceMethod("elementMetadata", "GAlignmentsList",
 setReplaceMethod("strand", "GAlignmentsList",
     GenomicRanges:::replaceStrandList
 )
+setReplaceMethod("strand", c("GAlignmentsList", "character"), 
+    function(x, ..., value)
+    {
+        if (length(value) > 1L)
+            stop("length(value) must be 1")
+        strand(x@unlistData) <- value
+        x
+    }
+)
 
 setReplaceMethod("seqinfo", "GAlignmentsList",
     GenomicRanges:::replaceSeqinfoList
@@ -203,29 +187,25 @@ setMethod("updateObject", "GAlignmentsList",
 ### Coercion.
 ###
 
-## FIXME (H.P. Feb 25, 2014):
-## This is broken if 'x' has empty elements:
-##   > grglist(GAlignmentsList(GAlignments("chr1", 20L, "10M", strand("+")),
-##                             GAlignments()))
-##   Error in relist(gr, .shiftPartition(...)) :
-##     shape of 'skeleton' is not compatible with 'NROW(flesh)'
 setMethod("grglist", "GAlignmentsList",
-    function(x, use.mcols=FALSE, order.as.in.query=FALSE, drop.D.ranges=FALSE,
-                ignore.strand=FALSE) 
+    function(x, use.mcols=FALSE, order.as.in.query=FALSE, 
+             drop.D.ranges=FALSE, ignore.strand=FALSE) 
     {
         if (!isTRUEorFALSE(use.mcols))
             stop("'use.mcols' must be TRUE or FALSE")
+        if (!identical(order.as.in.query, FALSE)) {
+            msg <- c("Starting with BioC 3.2, the 'order.as.in.query' ",
+                     "argument of the \"grglist\" method for ",
+                     "GAlignmentsList objects is deprecated and ignored.")
+            .Deprecated(msg=wmsg(msg))
+        }
+        if (!isTRUEorFALSE(ignore.strand))
+            stop("'ignore.strand' must be TRUE or FALSE")
         if (ignore.strand)
             strand(x@unlistData) <- "*"
-        rgl <- rglist(x@unlistData,
-                      order.as.in.query=order.as.in.query,
-                      drop.D.ranges=drop.D.ranges)
-        eltlen <- elementLengths(rgl)
-        gr <- GRanges(rep.int(seqnames(x@unlistData), eltlen), 
-                      ranges=unlist(rgl), 
-                      strand=rep.int(strand(x@unlistData), eltlen),
-                      seqinfo=seqinfo(x@unlistData))
-        ans <- relist(gr, .shiftPartition(x@partitioning, rgl@partitioning))
+        unlisted_x <- unlist(x, use.names=FALSE)
+        grl <- grglist(unlisted_x, drop.D.ranges=drop.D.ranges)
+        ans <- IRanges:::regroupBySupergroup(grl, x)
         if (use.mcols)
             mcols(ans) <- mcols(x)
         ans
@@ -246,7 +226,7 @@ setMethod("granges", "GAlignmentsList",
                       "is not parallel to 'x'.")
         rg <- range(grglist(x, ignore.strand=ignore.strand))
         is_one_to_one <- all(elementLengths(rg) == 1L)
-        if (!is_one_to_one) {
+        if (!is_one_to_one && all(width(x@partitioning) > 0)) {
             if (ignore.strand)
                 warning(msg)
             else
@@ -259,54 +239,29 @@ setMethod("granges", "GAlignmentsList",
     }
 )
 
-## FIXME (H.P. Feb 25, 2014):
-## This is broken if 'x' has empty elements:
-##   > rglist(GAlignmentsList(GAlignments("chr1", 20L, "10M", strand("+")),
-##                            GAlignments()))
-##   Error in relist(rgl@unlistData, partitioning) :
-##     shape of 'skeleton' is not compatible with 'NROW(flesh)'
 setMethod("rglist", "GAlignmentsList",
     function(x, use.mcols=FALSE, order.as.in.query=FALSE, drop.D.ranges=FALSE)
     {
         if (!isTRUEorFALSE(use.mcols))
             stop("'use.mcols' must be TRUE or FALSE")
-        rgl <- rglist(x@unlistData)
-        partitioning <- .shiftPartition(x@partitioning, rgl@partitioning) 
-        ans <- relist(rgl@unlistData, partitioning)
+        if (!identical(order.as.in.query, FALSE)) {
+            msg <- c("Starting with BioC 3.2, the 'order.as.in.query' ",
+                     "argument of the \"rglist\" method for ",
+                     "GAlignmentsList objects is deprecated and ignored.")
+            .Deprecated(msg=wmsg(msg))
+        }
+        unlisted_x <- unlist(x, use.names=FALSE)
+        rgl <- rglist(unlisted_x, drop.D.ranges=drop.D.ranges)
+        ans <- IRanges:::regroupBySupergroup(rgl, x)
         if (use.mcols)
             mcols(ans) <- mcols(x)
         ans
     }
 )
 
-## Adjust the widths of 'partition1' to accomodate the 
-## increased / decreased number of splits in 'partition2'.
-## The return value is a PartitioningByEnd object the same 
-## length as 'partition1'.
-.shiftPartition <- function(partition1, partition2)
-{
-    f <- rep(seq_along(partition1), width(partition1))
-    w <- sapply(split(width(partition2), f), sum)
-    if (any(w < 0))
-        stop("width of 'partition2' cannot be negative")
-    wdiff <- w - width(partition1)
-    PartitioningByEnd(end(partition1) + cumsum(wdiff), names=names(partition1))
-}
-
-## FIXME (H.P. Feb 25, 2014):
-## This is broken if 'x' has empty elements:
-##   > ranges(GAlignmentsList(GAlignments("chr1", 20L, "10M", strand("+")),
-##                            GAlignments()))
-##   Error in relist(unlist(rgl), .shiftPartition(...)) : 
-##     shape of 'skeleton' is not compatible with 'NROW(flesh)'
 setMethod("ranges", "GAlignmentsList",
     function(x) 
-    {
-        rgl <- rglist(x@unlistData)
-        lst <- relist(unlist(rgl), 
-                      .shiftPartition(x@partitioning, rgl@partitioning))
-        unlist(range(lst), use.names=FALSE)
-    }
+        unlist(range(rglist(x)), use.names=FALSE)
 )
 
 setAs("GAlignmentsList", "GRangesList", 
