@@ -258,16 +258,14 @@ flipQuery <- function(x, i)
 {
     if (!is(x, "GRangesList"))
         stop("'x' must be a GRangesList object")
-    if (missing(i))
-        i <- seq_len(length(x))
-    else
-        i <- IRanges:::normalizeSingleBracketSubscript(i, x)
-    xi <- x[i]
-    x[i] <- invertRleListStrand(revElements(xi))
+    i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+    xi <- extractROWS(x, i)
+    x <- replaceROWS(x, i, invertRleListStrand(revElements(xi)))
     xi_query.break <- mcols(xi)$query.break
     if (!is.null(xi_query.break)) {
         revxi_query.break <- elementLengths(xi) - xi_query.break
-        mcols(x)$query.break[i] <- revxi_query.break
+        mcols(x)$query.break <- replaceROWS(mcols(x)$query.break, i,
+                                            revxi_query.break)
     }
     x
 }
@@ -823,13 +821,13 @@ setMethod("extractSkippedExonRanks", "OverlapEncodings",
         stop("'elt_lens' must be an integer vector of the same length as 'x'")
     if (!is.integer(elt_lens))
         elt_lens <- as.integer(elt_lens)
-    if (IRanges:::anyMissingOrOutside(elt_lens, lower=0L))
+    if (S4Vectors:::anyMissingOrOutside(elt_lens, lower=0L))
         stop("'elt_lens' cannot contain NAs or negative values")
     x_elt_lens <- elementLengths(x)
     if (!all(elt_lens <= x_elt_lens))
         stop("'all(elt_lens <= elementLengths(x))' must be TRUE")
     offset <- cumsum(c(0L, x_elt_lens[-length(x_elt_lens)]))
-    ii <- IRanges:::fancy_mseq(elt_lens, offset=offset)
+    ii <- S4Vectors:::fancy_mseq(elt_lens, offset=offset)
     x@unlistData <- x@unlistData[ii]
     x@partitioning@end <- unname(cumsum(elt_lens))
     x
