@@ -3,6 +3,9 @@
 
 #define CALLMETHOD_DEF(fun, numArgs) {#fun, (DL_FUNC) &fun, numArgs}
 
+#define REGISTER_CCALLABLE(fun) \
+	R_RegisterCCallable("GenomicAlignments", #fun, (DL_FUNC) &fun)
+
 static const R_CallMethodDef callMethods[] = {
 
 /* cigar_utils.c */
@@ -14,6 +17,10 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(cigar_width, 3),
 	CALLMETHOD_DEF(cigar_narrow, 3),
 	CALLMETHOD_DEF(cigar_qnarrow, 3),
+
+/* mapping_methods.c */
+	CALLMETHOD_DEF(map_query_locs_to_ref_locs, 4),
+	CALLMETHOD_DEF(map_ref_locs_to_query_locs, 4),
 	CALLMETHOD_DEF(ref_locs_to_query_locs, 4),
 	CALLMETHOD_DEF(query_locs_to_ref_locs, 4),
 
@@ -28,6 +35,10 @@ static const R_CallMethodDef callMethods[] = {
 void R_init_GenomicAlignments(DllInfo *info)
 {
 	R_registerRoutines(info, NULL, callMethods, NULL, NULL);
+
+        /* cigar_utils.c */
+	REGISTER_CCALLABLE(_next_cigar_OP);
+
 	return;
 }
 
